@@ -172,6 +172,7 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.Pause)			<< "Pause\n";
 	*w << DisplayKeys(Key.Next)			<< "Next track\n";
 	*w << DisplayKeys(Key.Prev)			<< "Previous track\n";
+	*w << DisplayKeys(Key.Replay)			<< "Play current track from the beginning\n";
 	*w << DisplayKeys(Key.SeekForward)		<< "Seek forward\n";
 	*w << DisplayKeys(Key.SeekBackward)		<< "Seek backward\n";
 	*w << DisplayKeys(Key.VolumeDown)		<< "Decrease volume\n";
@@ -182,6 +183,7 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.ToggleMouse)		<< "Toggle mouse support\n";
 	*w << DisplayKeys(Key.ReverseSelection)		<< "Reverse selection\n";
 	*w << DisplayKeys(Key.DeselectAll)		<< "Deselect all items\n";
+	*w << DisplayKeys(Key.SelectAlbum)		<< "Select songs of album around cursor\n";
 	*w << DisplayKeys(Key.AddSelected)		<< "Add selected items to playlist/m3u file\n";
 	*w << "\n";
 	*w << DisplayKeys(Key.ToggleRepeat)		<< "Toggle repeat mode\n";
@@ -209,7 +211,7 @@ void Help::GetKeybindings()
 #	endif // HAVE_TAGLIB_H
 	*w << DisplayKeys(Key.ToggleDisplayMode)	<< "Toggle display mode\n";
 	*w << DisplayKeys(Key.ToggleInterface)		<< "Toggle user interface\n";
-	*w << DisplayKeys(Key.ToggleSeparatorsInPlaylist) << "Toggle displaying separators between albums in playlist\n";
+	*w << DisplayKeys(Key.ToggleSeparatorsInPlaylist) << "Toggle displaying separators between albums\n";
 	*w << DisplayKeys(Key.GoToPosition)		<< "Go to given position in current song (in % by default)\n";
 	*w << DisplayKeys(Key.SongInfo)			<< "Show song info\n";
 #	ifdef HAVE_CURL_CURL_H
@@ -230,6 +232,8 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.MvSongUp)			<< "Move item(s) up\n";
 	*w << DisplayKeys(Key.MvSongDown)		<< "Move item(s) down\n";
 	*w << DisplayKeys(Key.MoveTo)			<< "Move selected item(s) to cursor position\n";
+	*w << DisplayKeys(Key.MoveBefore)		<< "Move selected item(s) before cursor position\n";
+	*w << DisplayKeys(Key.MoveAfter)		<< "Move selected item(s) after cursor position\n";
 	*w << DisplayKeys(Key.Add)			<< "Add url/file/directory to playlist\n";
 #	ifdef HAVE_TAGLIB_H
 	*w << DisplayKeys(Key.EditTags)			<< "Edit song's tags\n";
@@ -270,8 +274,8 @@ void Help::GetKeybindings()
 	*w << "\n\n   " << fmtBold << "Keys - Media library\n -----------------------------------------\n" << fmtBoldEnd;
 	if (!Config.media_library_disable_two_column_mode)
 		*w << DisplayKeys(Key.MediaLibrary)	<< "Switch between two/three columns\n";
-	*w << DisplayKeys(&Key.VolumeDown[0], 1)	<< "Previous column\n";
-	*w << DisplayKeys(&Key.VolumeUp[0], 1)		<< "Next column\n";
+	*w << DisplayKeys(Key.PrevColumn)		<< "Previous column\n";
+	*w << DisplayKeys(Key.NextColumn)		<< "Next column\n";
 	*w << DisplayKeys(Key.Enter)			<< "Add to playlist and play song/album/artist's songs\n";
 	*w << DisplayKeys(Key.Space)			<< "Add to playlist song/album/artist's songs\n";
 #	ifdef HAVE_TAGLIB_H
@@ -281,8 +285,8 @@ void Help::GetKeybindings()
 	
 	
 	*w << "\n\n   " << fmtBold << "Keys - Playlist Editor\n -----------------------------------------\n" << fmtBoldEnd;
-	*w << DisplayKeys(&Key.VolumeDown[0], 1)	<< "Previous column\n";
-	*w << DisplayKeys(&Key.VolumeUp[0], 1)		<< "Next column\n";
+	*w << DisplayKeys(Key.PrevColumn)		<< "Previous column\n";
+	*w << DisplayKeys(Key.NextColumn)		<< "Next column\n";
 	*w << DisplayKeys(Key.Enter)			<< "Add item to playlist and play\n";
 	*w << DisplayKeys(Key.Space)			<< "Add to playlist/select item\n";
 #	ifdef HAVE_TAGLIB_H
@@ -301,6 +305,10 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Refetch lyrics\n";
 	
 	
+	*w << "\n\n   " << fmtBold << "Keys - Artist info\n -----------------------------------------\n" << fmtBoldEnd;
+	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Refetch artist info\n";
+	
+	
 #	ifdef HAVE_TAGLIB_H
 	*w << "\n\n   " << fmtBold << "Keys - Tiny tag editor\n -----------------------------------------\n" << fmtBoldEnd;
 	*w << DisplayKeys(Key.Enter)			<< "Edit tag\n";
@@ -312,8 +320,8 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.Enter)			<< "Perform operation on all/selected songs (middle column)\n";
 	*w << DisplayKeys(Key.Space)			<< "Switch to albums/directories view (left column)\n";
 	*w << DisplayKeys(Key.Space)			<< "Select/deselect song (right column)\n";
-	*w << DisplayKeys(&Key.VolumeDown[0], 1)	<< "Previous column\n";
-	*w << DisplayKeys(&Key.VolumeUp[0], 1)		<< "Next column\n";
+	*w << DisplayKeys(Key.PrevColumn)		<< "Previous column\n";
+	*w << DisplayKeys(Key.NextColumn)		<< "Next column\n";
 	*w << DisplayKeys(Key.GoToParentDir)		<< "Go to parent directory (left column, directories view)\n";
 #	endif // HAVE_TAGLIB_H
 	
