@@ -24,24 +24,16 @@
 #include "ncmpcpp.h"
 #include "mpdpp.h"
 #include "screen.h"
-
-#ifdef HAVE_CURL_CURL_H
-# include "curl/curl.h"
-#endif
+#include "lyrics_fetcher.h"
 
 class Lyrics : public Screen<Scrollpad>
 {
-	struct Plugin
-	{
-		const char *url;
-		const char *tag_open;
-		const char *tag_close;
-		bool (*not_found)(const std::string &);
-	};
-	
 	public:
-		Lyrics() : itsScrollBegin(0) { }
-		~Lyrics() { }
+		Lyrics() : ReloadNP(0),
+#		ifdef HAVE_CURL_CURL_H
+		isReadyToTake(0), isDownloadInProgress(0), itsFetcher(0),
+#		endif // HAVE_CURL_CURL_H
+		itsScrollBegin(0) { }
 		
 		virtual void Resize();
 		virtual void SwitchTo();
@@ -58,42 +50,34 @@ class Lyrics : public Screen<Scrollpad>
 		virtual List *GetList() { return 0; }
 		
 		void Edit();
-		void FetchAgain();
-		
-		static bool Reload;
-		
+		void Refetch();
 #		ifdef HAVE_CURL_CURL_H
-		static const char *GetPluginName(int offset);
-		
-		static const unsigned DBs;
+		void ToggleFetcher();
 #		endif // HAVE_CURL_CURL_H
 		
+		bool ReloadNP;
+		
 	protected:
 		virtual void Init();
 		
 	private:
-		std::string itsFilenamePath;
+		void Load();
 		
-		static const std::string Folder;
+		void SetFilename();
+		std::string itsFilename;
+		std::string itsFolder;
 		
 #		ifdef HAVE_CURL_CURL_H
-		static void *Get(void *);
+		void *Download();
+		static void *DownloadWrapper(void *);
+		
+		void Save(const std::string &lyrics);
 		
-#		ifdef HAVE_PTHREAD_H
 		void Take();
-#		endif // HAVE_PTHREAD_H
-		
-		static const Plugin *ChoosePlugin(int);
-		static bool LyricsPlugin_NotFound(const std::string &);
-		
-		static bool Ready;
-		
-#		ifdef HAVE_PTHREAD_H
-		static pthread_t *Downloader;
-#		endif // HAVE_PTHREAD_H
-		
-		static const char *PluginsList[];
-		static const Plugin LyricsPlugin;
+		bool isReadyToTake;
+		bool isDownloadInProgress;
+		pthread_t itsDownloader;
+		LyricsFetcher **itsFetcher;
 #		endif // HAVE_CURL_CURL_H
 		
 		size_t itsScrollBegin;
