@@ -28,10 +28,10 @@ class MediaLibrary : public Screen<Window>
 {
 	struct SearchConstraints
 	{
-		SearchConstraints(const std::string &artist, const std::string &album, const std::string &year) : Artist(artist), Album(album), Year(year) { }
+		SearchConstraints(const std::string &tag, const std::string &album, const std::string &year) : PrimaryTag(tag), Album(album), Year(year) { }
 		SearchConstraints(const std::string &album, const std::string &year) : Album(album), Year(year) { }
 		
-		std::string Artist;
+		std::string PrimaryTag;
 		std::string Album;
 		std::string Year;
 	};
@@ -56,6 +56,7 @@ class MediaLibrary : public Screen<Window>
 		virtual bool isTabbable() { return true; }
 		
 		virtual MPD::Song *CurrentSong();
+		virtual MPD::Song *GetSong(size_t pos) { return w == Songs ? &Songs->at(pos) : 0; }
 		
 		virtual bool allowsSelection() { return true; }
 		virtual void ReverseSelection();
@@ -85,6 +86,7 @@ class MediaLibrary : public Screen<Window>
 		
 		static std::string AlbumToString(const SearchConstraints &, void *);
 		static void DisplayAlbums(const SearchConstraints &, void *, Menu<SearchConstraints> *);
+		static void DisplayPrimaryTags(const std::string &artist, void *, Menu<std::string> *menu);
 		
 		static bool SortSongsByTrack(MPD::Song *, MPD::Song *);
 		static bool SortAllTracks(MPD::Song *, MPD::Song *);
