@@ -95,6 +95,7 @@ namespace MPD
 			unsigned Version() const;
 			
 			void SetIdleEnabled(bool val) { isIdleEnabled = val; }
+			void BlockIdle(bool val) { itsIdleBlocked = val; }
 			bool SupportsIdle() const { return supportsIdle; }
 			void OrderDataFetching() { hasData = 1; }
 			int GetFD() const { return itsFD; }
@@ -183,7 +184,8 @@ namespace MPD
 			bool CommitCommandsList();
 			
 			bool DeletePlaylist(const std::string &);
-			bool SavePlaylist(const std::string &);
+			bool LoadPlaylist(const std::string &name);
+			int SavePlaylist(const std::string &);
 			void ClearPlaylist(const std::string &);
 			void AddToPlaylist(const std::string &, const Song &);
 			void AddToPlaylist(const std::string &, const std::string &);
@@ -227,6 +229,7 @@ namespace MPD
 			int itsFD;
 			bool isIdle;
 			bool isIdleEnabled;
+			bool itsIdleBlocked;
 			bool supportsIdle;
 			bool hasData;
 			
