@@ -47,7 +47,8 @@
 #include "search_engine.h"
 #include "settings.h"
 #include "song.h"
-#include "info.h"
+#include "song_info.h"
+#include "lastfm.h"
 #include "outputs.h"
 #include "status.h"
 #include "tag_editor.h"
@@ -139,10 +140,13 @@ namespace
 		mySearcher->hasToBeResized = 1;
 		myLibrary->hasToBeResized = 1;
 		myPlaylistEditor->hasToBeResized = 1;
-		myInfo->hasToBeResized = 1;
 		myLyrics->hasToBeResized = 1;
 		mySelectedItemsAdder->hasToBeResized = 1;
 		
+#		ifdef HAVE_CURL_CURL_H
+		myLastfm->hasToBeResized = 1;
+#		endif // HAVE_CURL_CURL_H
+		
 #		ifdef HAVE_TAGLIB_H
 		myTinyTagEditor->hasToBeResized = 1;
 		myTagEditor->hasToBeResized = 1;
@@ -261,16 +265,17 @@ int main(int argc, char *argv[])
 	
 	bool real_statusbar_visibility = Config.statusbar_visibility;
 	
+	if (!Config.titles_visibility)
+		wattron(stdscr, COLOR_PAIR(Config.main_color));
+	
 	if (Config.new_design)
 		Config.statusbar_visibility = 0;
 	
 	SetWindowsDimensions(header_height, footer_start_y, footer_height);
 	
+	wHeader = new Window(0, 0, COLS, header_height, "", Config.header_color, brNone);
 	if (Config.header_visibility || Config.new_design)
-	{
-		wHeader = new Window(0, 0, COLS, header_height, "", Config.header_color, brNone);
 		wHeader->Display();
-	}
 	
 	wFooter = new Window(0, footer_start_y, COLS, footer_height, "", Config.statusbar_color, brNone);
 	wFooter->SetTimeout(ncmpcpp_window_timeout);
@@ -552,7 +557,13 @@ int main(int argc, char *argv[])
 			design_changed = 1;
 			resize_screen();
 		}
-		else if (Keypressed(input, Key.GoToParentDir))
+		else if (Keypressed(input, Key.GoToParentDir)
+		     &&  (myScreen == myBrowser
+#			ifdef HAVE_TAGLIB_H
+		        || myScreen == myTagEditor
+#			endif // HAVE_TAGLIB_H
+			 )
+			)
 		{
 			if (myScreen == myBrowser && myBrowser->CurrentDir() != "/")
 			{
@@ -575,43 +586,61 @@ int main(int argc, char *argv[])
 		{
 			myScreen->SpacePressed();
 		}
-		else if (Keypressed(input, Key.VolumeUp))
+		else if (Keypressed(input, Key.PrevColumn)
+		     &&  (myScreen == myLibrary
+		       || myScreen == myPlaylistEditor
+#		ifdef HAVE_TAGLIB_H
+		       || myScreen == myTagEditor
+#		endif // HAVE_TAGLIB_H)
+			 )
+			)
 		{
-			if (myScreen == myLibrary && input == Key.VolumeUp[0])
-			{
-				myLibrary->NextColumn();
-			}
-			else if (myScreen == myPlaylistEditor && input == Key.VolumeUp[0])
-			{
-				myPlaylistEditor->NextColumn();
-			}
-#			ifdef HAVE_TAGLIB_H
-			else if (myScreen == myTagEditor && input == Key.VolumeUp[0])
-			{
-				myTagEditor->NextColumn();
-			}
-#			endif // HAVE_TAGLIB_H
-			else
-				Mpd.SetVolume(Mpd.GetVolume()+1);
-		}
-		else if (Keypressed(input, Key.VolumeDown))
-		{
-			if (myScreen == myLibrary && input == Key.VolumeDown[0])
+			if (myScreen == myLibrary)
 			{
 				myLibrary->PrevColumn();
 			}
-			else if (myScreen == myPlaylistEditor && input == Key.VolumeDown[0])
+			else if (myScreen == myPlaylistEditor)
 			{
 				myPlaylistEditor->PrevColumn();
 			}
 #			ifdef HAVE_TAGLIB_H
-			else if (myScreen == myTagEditor && input == Key.VolumeDown[0])
+			else if (myScreen == myTagEditor)
 			{
 				myTagEditor->PrevColumn();
 			}
 #			endif // HAVE_TAGLIB_H
-			else
-				Mpd.SetVolume(Mpd.GetVolume()-1);
+		}
+		else if (Keypressed(input, Key.NextColumn)
+		     &&  (myScreen == myLibrary
+		       || myScreen == myPlaylistEditor
+#		ifdef HAVE_TAGLIB_H
+		       || myScreen == myTagEditor
+#		endif // HAVE_TAGLIB_H)
+			 )
+			)
+		{
+			if (myScreen == myLibrary)
+			{
+				myLibrary->NextColumn();
+			}
+			else if (myScreen == myPlaylistEditor)
+			{
+				myPlaylistEditor->NextColumn();
+			}
+#			ifdef HAVE_TAGLIB_H
+			else if (myScreen == myTagEditor)
+			{
+				myTagEditor->NextColumn();
+			}
+#			endif // HAVE_TAGLIB_H
+		}
+		else if (Keypressed(input, Key.VolumeUp))
+		{
+			Mpd.SetVolume(Mpd.GetVolume()+1);
+		}
+		else if (Keypressed(input, Key.VolumeDown))
+		{
+			Mpd.SetVolume(Mpd.GetVolume()-1);
 		}
 		else if (Keypressed(input, Key.Delete))
 		{
@@ -659,7 +688,7 @@ int main(int argc, char *argv[])
 				}
 			}
 			else if (
-				 (myScreen == myBrowser && !myBrowser->Main()->Empty() && myBrowser->Main()->Current().type == itPlaylist)
+				 (myScreen == myBrowser && !myBrowser->Main()->Empty() && myBrowser->CurrentDir() == "/" && myBrowser->Main()->Current().type == itPlaylist)
 			||       (myScreen->ActiveWindow() == myPlaylistEditor->Playlists)
 				)
 			{
@@ -691,7 +720,7 @@ int main(int argc, char *argv[])
 					myPlaylistEditor->Playlists->Clear(); // make playlists list update itself
 			}
 #			ifndef WIN32
-			else if (myScreen == myBrowser && !myBrowser->Main()->Empty() && myBrowser->Main()->Current().type != itPlaylist)
+			else if (myScreen == myBrowser && !myBrowser->Main()->Empty())
 			{
 				if (!myBrowser->isLocal())
 					CHECK_MPD_MUSIC_DIR;
@@ -713,7 +742,10 @@ int main(int argc, char *argv[])
 				
 				std::string name = item.type == itSong ? item.song->GetName() : item.name;
 				LockStatusbar();
-				Statusbar() << "Delete " << (item.type == itSong ? "file" : "directory") << " \"" << Shorten(TO_WSTRING(name), COLS-30) << "\" ? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
+				if (myBrowser->Main()->hasSelected())
+					Statusbar() << "Delete selected items ? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
+				else
+					Statusbar() << "Delete " << (item.type == itSong ? "file" : item.type == itDirectory ? "directory" : "playlist") << " \"" << Shorten(TO_WSTRING(name), COLS-30) << "\" ? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
 				wFooter->Refresh();
 				int answer = 0;
 				do
@@ -725,28 +757,35 @@ int main(int argc, char *argv[])
 				UnlockStatusbar();
 				if (answer == 'y')
 				{
-					std::string path;
-					if (!myBrowser->isLocal())
-						path = Config.mpd_music_dir;
-					path += item.type == itSong ? item.song->GetFile() : item.name;
-					
-					if (item.type == itDirectory)
-						myBrowser->ClearDirectory(path);
-					
-					if (remove(path.c_str()) == 0)
+					std::vector<size_t> list;
+					myBrowser->Main()->GetSelected(list);
+					if (list.empty())
+						list.push_back(myBrowser->Main()->Choice());
+					bool success = 1;
+					for (size_t i = 0; i < list.size(); ++i)
+					{
+						const MPD::Item &it = (*myBrowser->Main())[list[i]];
+						name = it.type == itSong ? it.song->GetName() : it.name;
+						if (myBrowser->DeleteItem(it))
+						{
+							static const char msg[] = "\"%s\" deleted!";
+							ShowMessage(msg, Shorten(TO_WSTRING(name), COLS-static_strlen(msg)).c_str());
+						}
+						else
+						{
+							static const char msg[] = "Couldn't remove \"%s\": %s";
+							ShowMessage(msg, Shorten(TO_WSTRING(name), COLS-static_strlen(msg)-25).c_str(), strerror(errno));
+							success = 0;
+							break;
+						}
+					}
+					if (success)
 					{
-						static const char msg[] = "\"%s\" deleted!";
-						ShowMessage(msg, Shorten(TO_WSTRING(name), COLS-static_strlen(msg)).c_str());
 						if (!myBrowser->isLocal())
 							Mpd.UpdateDirectory(myBrowser->CurrentDir());
 						else
 							myBrowser->GetDirectory(myBrowser->CurrentDir());
 					}
-					else
-					{
-						static const char msg[] = "Couldn't remove \"%s\": %s";
-						ShowMessage(msg, Shorten(TO_WSTRING(name), COLS-static_strlen(msg)-25).c_str(), strerror(errno));
-					}
 				}
 				else
 					ShowMessage("Aborted!");
@@ -788,6 +827,14 @@ int main(int argc, char *argv[])
 				}
 			}
 		}
+		else if (Keypressed(input, Key.Replay))
+		{
+			if (Mpd.isPlaying())
+			{
+				Mpd.Seek(0);
+				UpdateStatusImmediately = 1;
+			}
+		}
 		else if (Keypressed(input, Key.Prev))
 		{
 			Mpd.Prev();
@@ -826,37 +873,41 @@ int main(int argc, char *argv[])
 					if (Mpd.GetErrorMessage().empty())
 						ShowMessage("Filtered items added to playlist \"%s\"", playlist_name.c_str());
 				}
-				else if (Mpd.SavePlaylist(real_playlist_name))
-				{
-					ShowMessage("Playlist saved as: %s", playlist_name.c_str());
-					if (myPlaylistEditor->Main()) // check if initialized
-						myPlaylistEditor->Playlists->Clear(); // make playlist's list update itself
-				}
 				else
 				{
-					LockStatusbar();
-					Statusbar() << "Playlist already exists, overwrite: " << playlist_name << " ? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
-					wFooter->Refresh();
-					int answer = 0;
-					while (answer != 'y' && answer != 'n')
+					int result = Mpd.SavePlaylist(real_playlist_name);
+					if (result == MPD_ERROR_SUCCESS)
 					{
-						TraceMpdStatus();
-						wFooter->ReadKey(answer);
+						ShowMessage("Playlist saved as: %s", playlist_name.c_str());
+						if (myPlaylistEditor->Main()) // check if initialized
+							myPlaylistEditor->Playlists->Clear(); // make playlist's list update itself
 					}
-					UnlockStatusbar();
-					
-					if (answer == 'y')
+					else if (result == MPD_SERVER_ERROR_EXIST)
 					{
-						Mpd.DeletePlaylist(real_playlist_name);
-						if (Mpd.SavePlaylist(real_playlist_name))
-							ShowMessage("Playlist overwritten!");
+						LockStatusbar();
+						Statusbar() << "Playlist already exists, overwrite: " << playlist_name << " ? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
+						wFooter->Refresh();
+						int answer = 0;
+						while (answer != 'y' && answer != 'n')
+						{
+							TraceMpdStatus();
+							wFooter->ReadKey(answer);
+						}
+						UnlockStatusbar();
+						
+						if (answer == 'y')
+						{
+							Mpd.DeletePlaylist(real_playlist_name);
+							if (Mpd.SavePlaylist(real_playlist_name) == MPD_ERROR_SUCCESS)
+								ShowMessage("Playlist overwritten!");
+						}
+						else
+							ShowMessage("Aborted!");
+						if (myPlaylistEditor->Main()) // check if initialized
+							myPlaylistEditor->Playlists->Clear(); // make playlist's list update itself
+						if (myScreen == myPlaylist)
+							myPlaylist->EnableHighlighting();
 					}
-					else
-						ShowMessage("Aborted!");
-					if (myPlaylistEditor->Main()) // check if initialized
-						myPlaylistEditor->Playlists->Clear(); // make playlist's list update itself
-					if (myScreen == myPlaylist)
-						myPlaylist->EnableHighlighting();
 				}
 			}
 			if (myBrowser->Main()
@@ -1129,7 +1180,8 @@ int main(int argc, char *argv[])
 				wFooter->SetTimeout(ncmpcpp_window_timeout);
 			}
 		}
-		else if (Keypressed(input, Key.MoveTo) && myScreen == myPlaylist)
+		else if ((Keypressed(input, Key.MoveTo) || Keypressed(input, Key.MoveBefore)
+					|| Keypressed(input, Key.MoveAfter)) && myScreen == myPlaylist)
 		{
 			CHECK_PLAYLIST_FOR_FILTERING;
 			if (!myPlaylist->Items->hasSelected())
@@ -1139,13 +1191,18 @@ int main(int argc, char *argv[])
 			}
 			Playlist::BlockUpdate = 1;
 			size_t pos = myPlaylist->Items->Choice();
-			 // if cursor is at the last item, break convention and move at the end of playlist
-			if (pos == myPlaylist->Items->Size()-1)
-				pos++;
 			std::vector<size_t> list;
 			myPlaylist->Items->GetSelected(list);
 			if (pos >= list.front() && pos <= list.back())
 				continue;
+			if (Keypressed(input, Key.MoveTo))
+			{
+				 // if cursor is at the last item, break convention and move at the end of playlist
+				if (pos == myPlaylist->Items->Size()-1)
+					pos++;
+			}
+			else if (Keypressed(input, Key.MoveAfter))
+				pos++;
 			int diff = pos-list.front();
 			Mpd.StartCommandsList();
 			if (diff > 0)
@@ -1181,13 +1238,21 @@ int main(int argc, char *argv[])
 			UnlockStatusbar();
 			if (!path.empty())
 			{
+				Statusbar() << "Adding...";
+				wFooter->Refresh();
 				if (myScreen == myPlaylistEditor)
 				{
 					Mpd.AddToPlaylist(myPlaylistEditor->Playlists->Current(), path);
 					myPlaylistEditor->Content->Clear(); // make it refetch content of playlist
 				}
 				else
-					Mpd.Add(path);
+				{
+					static const char lastfm_url[] = "lastfm://";
+					if (path.compare(0, static_strlen(lastfm_url), lastfm_url) == 0)
+						Mpd.LoadPlaylist(path);
+					else
+						Mpd.Add(path);
+				}
 				UpdateStatusImmediately = 1;
 			}
 		}
@@ -1290,7 +1355,7 @@ int main(int argc, char *argv[])
 				if (Config.columns_in_playlist)
 				{
 					myPlaylist->Items->SetItemDisplayer(Display::SongsInColumns);
-					myPlaylist->Items->SetTitle(Display::Columns());
+					myPlaylist->Items->SetTitle(Config.titles_visibility ? Display::Columns() : "");
 					myPlaylist->Items->SetGetStringFunction(Playlist::SongInColumnsToString);
 				}
 				else
@@ -1304,7 +1369,7 @@ int main(int argc, char *argv[])
 			{
 				Config.columns_in_browser = !Config.columns_in_browser;
 				ShowMessage("Browser display mode: %s", Config.columns_in_browser ? "Columns" : "Classic");
-				myBrowser->Main()->SetTitle(Config.columns_in_browser ? Display::Columns() : "");
+				myBrowser->Main()->SetTitle(Config.columns_in_browser && Config.titles_visibility ? Display::Columns() : "");
 				
 			}
 			else if (myScreen == mySearcher)
@@ -1312,7 +1377,7 @@ int main(int argc, char *argv[])
 				Config.columns_in_search_engine = !Config.columns_in_search_engine;
 				ShowMessage("Search engine display mode: %s", Config.columns_in_search_engine ? "Columns" : "Classic");
 				if (mySearcher->Main()->Size() > SearchEngine::StaticOptions)
-					mySearcher->Main()->SetTitle(Config.columns_in_search_engine ? Display::Columns() : "");
+					mySearcher->Main()->SetTitle(Config.columns_in_search_engine && Config.titles_visibility ? Display::Columns() : "");
 			}
 		}
 		else if (Keypressed(input, Key.ToggleSeparatorsInPlaylist))
@@ -1323,12 +1388,7 @@ int main(int argc, char *argv[])
 #		ifdef HAVE_CURL_CURL_H
 		else if (Keypressed(input, Key.ToggleLyricsDB))
 		{
-			const char *current_lyrics_plugin = Lyrics::GetPluginName(++Config.lyrics_db);
-			if (!current_lyrics_plugin)
-			{
-				current_lyrics_plugin = Lyrics::GetPluginName(Config.lyrics_db = 0);
-			}
-			ShowMessage("Using lyrics database: %s", current_lyrics_plugin);
+			myLyrics->ToggleFetcher();
 		}
 #		endif // HAVE_CURL_CURL_H
 		else if (Keypressed(input, Key.ToggleAutoCenter))
@@ -1437,7 +1497,7 @@ int main(int argc, char *argv[])
 			{
 				myTinyTagEditor->SwitchTo();
 			}
-			else if (myScreen->ActiveWindow() == myLibrary->Artists)
+			else if (myScreen->ActiveWindow() == myLibrary->Artists && !myLibrary->Artists->Empty())
 			{
 				LockStatusbar();
 				Statusbar() << fmtBold << IntoStr(Config.media_lib_primary_tag) << fmtBoldEnd << ": ";
@@ -1476,7 +1536,7 @@ int main(int argc, char *argv[])
 					FreeSongList(list);
 				}
 			}
-			else if (myScreen->ActiveWindow() == myLibrary->Albums)
+			else if (myScreen->ActiveWindow() == myLibrary->Albums && !myLibrary->Albums->Empty())
 			{
 				LockStatusbar();
 				Statusbar() << fmtBold << "Album: " << fmtBoldEnd;
@@ -1515,7 +1575,9 @@ int main(int argc, char *argv[])
 					}
 				}
 			}
-			else if (myScreen->ActiveWindow() == myTagEditor->Dirs)
+			else if (myScreen->ActiveWindow() == myTagEditor->Dirs
+			     &&  !myTagEditor->Dirs->Empty()
+			     &&  myTagEditor->Dirs->Choice() > 0)
 			{
 				std::string old_dir = myTagEditor->Dirs->Current().first;
 				LockStatusbar();
@@ -1545,7 +1607,7 @@ int main(int argc, char *argv[])
 			{
 				myLyrics->Edit();
 			}
-			if (myScreen == myBrowser && myBrowser->Main()->Current().type == itDirectory)
+			if (myScreen == myBrowser && !myBrowser->Main()->Empty() && myBrowser->Main()->Current().type == itDirectory)
 			{
 				std::string old_dir = myBrowser->Main()->Current().name;
 				LockStatusbar();
@@ -1578,7 +1640,15 @@ int main(int argc, char *argv[])
 					}
 				}
 			}
-			else if (myScreen->ActiveWindow() == myPlaylistEditor->Playlists || (myScreen == myBrowser && myBrowser->Main()->Current().type == itPlaylist))
+			else if (
+				 (myScreen->ActiveWindow() == myPlaylistEditor->Playlists
+			       && !myPlaylistEditor->Playlists->Empty()
+				 )
+			     ||  (myScreen == myBrowser
+			       && !myBrowser->Main()->Empty()
+			       && myBrowser->Main()->Current().type == itPlaylist
+				 )
+				)
 			{
 				std::string old_name = myScreen->ActiveWindow() == myPlaylistEditor->Playlists ? myPlaylistEditor->Playlists->Current() : myBrowser->Main()->Current().name;
 				LockStatusbar();
@@ -1694,6 +1764,43 @@ int main(int argc, char *argv[])
 				ShowMessage("Items deselected!");
 			}
 		}
+		else if (Keypressed(input, Key.SelectAlbum))
+		{
+			if (myScreen->allowsSelection())
+			{
+				if (List *mList = myScreen->GetList())
+				{
+					size_t pos = mList->Choice();
+					if (MPD::Song *s = myScreen->GetSong(pos))
+					{
+						std::string album = s->GetAlbum();
+						
+						// select song under cursor
+						mList->Select(pos, 1);
+						// go up
+						while (pos > 0)
+						{
+							s = myScreen->GetSong(--pos);
+							if (!s || s->GetAlbum() != album)
+								break;
+							else
+								mList->Select(pos, 1);
+						}
+						// go down
+						pos = mList->Choice();
+						while (pos < mList->Size() - 1)
+						{
+							s = myScreen->GetSong(++pos);
+							if (!s || s->GetAlbum() != album)
+								break;
+							else
+								mList->Select(pos, 1);
+						}
+						ShowMessage("Album around cursor position selected.");
+					}
+				}
+			}
+		}
 		else if (Keypressed(input, Key.AddSelected))
 		{
 			mySelectedItemsAdder->SwitchTo();
@@ -1701,6 +1808,28 @@ int main(int argc, char *argv[])
 		else if (Keypressed(input, Key.Crop))
 		{
 			CHECK_PLAYLIST_FOR_FILTERING;
+			if (Config.ask_before_clearing_main_playlist)
+			{
+				LockStatusbar();
+				Statusbar() << "Do you really want to crop playlist";
+				if (myScreen->ActiveWindow() == myPlaylistEditor->Content)
+					*wFooter << " \"" << myPlaylistEditor->Playlists->Current() << "\"";
+				*wFooter << " ? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
+				wFooter->Refresh();
+				int answer = 0;
+				do
+				{
+					TraceMpdStatus();
+					wFooter->ReadKey(answer);
+				}
+				while (answer != 'y' && answer != 'n');
+				UnlockStatusbar();
+				if (answer != 'y')
+				{
+					ShowMessage("Aborted!");
+					continue;
+				}
+			}
 			if (myPlaylist->Items->hasSelected())
 			{
 				Mpd.StartCommandsList();
@@ -1852,7 +1981,11 @@ int main(int argc, char *argv[])
 				if (myScreen == myPlaylist)
 					myPlaylist->EnableHighlighting();
 			}
-			else if (myScreen == myHelp || myScreen == myLyrics || myScreen == myInfo)
+			else if (myScreen == myHelp || myScreen == myLyrics
+#			ifdef HAVE_CURL_CURL_H
+			     || myScreen == myLastfm
+#			endif // HAVE_CURL_CURL_H
+				)
 			{
 				LockStatusbar();
 				Statusbar() << "Find: ";
@@ -1972,7 +2105,7 @@ int main(int argc, char *argv[])
 				{
 					Config.media_lib_primary_tag = new_tagitem;
 					std::string item_type = IntoStr(Config.media_lib_primary_tag);
-					myLibrary->Artists->SetTitle(item_type + "s");
+					myLibrary->Artists->SetTitle(Config.titles_visibility ? item_type + "s" : "");
 					myLibrary->Artists->Reset();
 					ToLower(item_type);
 					if (myLibrary->Columns() == 2)
@@ -1980,7 +2113,7 @@ int main(int argc, char *argv[])
 						myLibrary->Songs->Clear();
 						myLibrary->Albums->Reset();
 						myLibrary->Albums->Clear();
-						myLibrary->Albums->SetTitle("Albums (sorted by " + item_type + ")");
+						myLibrary->Albums->SetTitle(Config.titles_visibility ? "Albums (sorted by " + item_type + ")" : "");
 						myLibrary->Albums->Display();
 					}
 					else
@@ -1993,17 +2126,38 @@ int main(int argc, char *argv[])
 			}
 			else if (myScreen == myLyrics)
 			{
-				myLyrics->FetchAgain();
+				myLyrics->Refetch();
 			}
+#			ifdef HAVE_CURL_CURL_H
+			else if (myScreen == myLastfm)
+			{
+				myLastfm->Refetch();
+			}
+#			endif // HAVE_CURL_CURL_H
 		}
 		else if (Keypressed(input, Key.SongInfo))
 		{
-			myInfo->GetSong();
+			mySongInfo->SwitchTo();
 		}
 #		ifdef HAVE_CURL_CURL_H
 		else if (Keypressed(input, Key.ArtistInfo))
 		{
-			myInfo->GetArtist();
+			if (myScreen == myLastfm || myLastfm->isDownloading())
+			{
+				myLastfm->SwitchTo();
+				continue;
+			}
+			
+			std::string artist;
+			MPD::Song *s = myScreen->CurrentSong();
+			
+			if (s)
+				artist = s->GetArtist();
+			else if (myScreen == myLibrary && myLibrary->Main() == myLibrary->Artists && !myLibrary->Artists->Empty())
+				artist = myLibrary->Artists->Current();
+			
+			if (!artist.empty() && myLastfm->SetArtistInfoArgs(artist, Config.lastfm_preferred_language))
+				myLastfm->SwitchTo();
 		}
 #		endif // HAVE_CURL_CURL_H
 		else if (Keypressed(input, Key.Lyrics))
