@@ -47,14 +47,16 @@ Menu< std::pair<std::string, MPD::Song::GetFunction> > *Playlist::SortDialog = 0
 
 void Playlist::Init()
 {
-	Items = new Menu<MPD::Song>(0, MainStartY, COLS, MainHeight, Config.columns_in_playlist ? Display::Columns() : "", Config.main_color, brNone);
+	static Display::ScreenFormat sf = { this, &Config.song_list_format };
+	
+	Items = new Menu<MPD::Song>(0, MainStartY, COLS, MainHeight, Config.columns_in_playlist && Config.titles_visibility ? Display::Columns() : "", Config.main_color, brNone);
 	Items->CyclicScrolling(Config.use_cyclic_scrolling);
 	Items->CenteredCursor(Config.centered_cursor);
 	Items->HighlightColor(Config.main_highlight_color);
 	Items->SetSelectPrefix(&Config.selected_item_prefix);
 	Items->SetSelectSuffix(&Config.selected_item_suffix);
 	Items->SetItemDisplayer(Config.columns_in_playlist ? Display::SongsInColumns : Display::Songs);
-	Items->SetItemDisplayerUserData(&Config.song_list_format);
+	Items->SetItemDisplayerUserData(&sf);
 	Items->SetGetStringFunction(Config.columns_in_playlist ? SongInColumnsToString : SongToString);
 	Items->SetGetStringFunctionUserData(&Config.song_list_format);
 	
@@ -116,7 +118,7 @@ void Playlist::Resize()
 {
 	Items->Resize(COLS, MainHeight);
 	Items->MoveTo(0, MainStartY);
-	Items->SetTitle(Config.columns_in_playlist ? Display::Columns() : "");
+	Items->SetTitle(Config.columns_in_playlist && Config.titles_visibility ? Display::Columns() : "");
 	if (w == SortDialog) // if sorting window is active, playlist needs refreshing
 		Items->Display();
 	
@@ -275,7 +277,7 @@ void Playlist::MouseButtonPressed(MEVENT me)
 
 MPD::Song *Playlist::CurrentSong()
 {
-	return !Items->Empty() ? &Items->Current() : 0;
+	return w == Items && !Items->Empty() ? &Items->Current() : 0;
 }
 
 void Playlist::GetSelectedSongs(MPD::SongList &v)
@@ -491,7 +493,8 @@ bool Playlist::Add(const MPD::SongList &l, bool play, int position)
 			if (Mpd.AddSong(**j, position) < 0)
 				break;
 	}
-	Mpd.CommitCommandsList();
+	if (!Mpd.CommitCommandsList())
+		return false;
 	
 	if (play && old_playlist_size < Items->Size())
 		Mpd.Play(old_playlist_size);
