@@ -103,7 +103,7 @@ namespace MPD
 			Song &operator=(const Song &);
 			
 			static std::string ShowTime(int);
-			static void ValidateFormat(const std::string &type, const std::string &format);
+			static bool isFormatOk(const std::string &type, const std::string &format);
 			
 		private:
 			void SetHashAndSlash();
