@@ -205,6 +205,11 @@ void NcmpcppErrorCallback(MPD::Connection *, int errorid, const char *msg, void
 			ShowMessage("Password accepted!");
 		wFooter->SetGetStringHelper(StatusbarGetStringHelper);
 	}
+	else if ((errorid >> 8) == MPD_SERVER_ERROR_NO_EXIST && myScreen == myBrowser)
+	{
+		myBrowser->GetDirectory(PathGoDownOneLevel(myBrowser->CurrentDir()));
+		myBrowser->Refresh();
+	}
 	else
 		ShowMessage("%s", msg);
 }
@@ -352,11 +357,7 @@ void NcmpcppStatusChanged(MPD::Connection *, MPD::StatusChanges changed, void *)
 			{
 				WindowTitle("ncmpc++ ver. "VERSION);
 				if (!block_progressbar_update)
-				{
-					*wFooter << Config.progressbar_color;
-					mvwhline(wFooter->Raw(), 0, 0, 0, wFooter->GetWidth());
-					*wFooter << clEnd;
-				}
+					DrawProgressbar(0, 0);
 				Playlist::ReloadRemaining = 1;
 				myPlaylist->NowPlaying = -1;
 				if (Config.new_design)
@@ -407,7 +408,7 @@ void NcmpcppStatusChanged(MPD::Connection *, MPD::StatusChanges changed, void *)
 				myPlaylist->Items->Highlight(myPlaylist->NowPlaying);
 			
 			if (Config.now_playing_lyrics && myScreen == myLyrics && Global::myOldScreen == myPlaylist)
-				Lyrics::Reload = 1;
+				myLyrics->ReloadNP = 1;
 		}
 		Playlist::ReloadRemaining = 1;
 		playing_song_scroll_begin = 0;
@@ -652,12 +653,21 @@ Window &Statusbar()
 
 void DrawProgressbar(unsigned elapsed, unsigned time)
 {
-	unsigned howlong = time ? wFooter->GetWidth()*elapsed/time : 0;
+	unsigned pb_width = wFooter->GetWidth();
+	unsigned howlong = time ? pb_width*elapsed/time : 0;
 	*wFooter << fmtBold << Config.progressbar_color;
-	mvwhline(wFooter->Raw(), 0, 0, 0, wFooter->GetWidth());
+	if (Config.progressbar[2] != '\0')
+	{
+		wFooter->GotoXY(0, 0);
+		for (unsigned i = 0; i < pb_width; ++i)
+			*wFooter << Config.progressbar[2];
+		wFooter->GotoXY(0, 0);
+	}
+	else
+		mvwhline(wFooter->Raw(), 0, 0, 0, pb_width);
 	if (time)
 	{
-		unsigned pb_width = std::min(size_t(howlong), wFooter->GetWidth());
+		pb_width = std::min(size_t(howlong), wFooter->GetWidth());
 		for (unsigned i = 0; i < pb_width; ++i)
 			*wFooter << Config.progressbar[0];
 		if (howlong < wFooter->GetWidth())
