@@ -36,7 +36,7 @@
 #include "charset.h"
 #include "display.h"
 #include "global.h"
-#include "info.h"
+#include "song_info.h"
 #include "playlist.h"
 
 using Global::MainHeight;
@@ -64,14 +64,14 @@ void TagEditor::Init()
 {
 	SetDimensions();
 	
-	Albums = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, "Albums", Config.main_color, brNone);
+	Albums = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, Config.titles_visibility ? "Albums" : "", Config.main_color, brNone);
 	Albums->HighlightColor(Config.active_column_color);
 	Albums->CyclicScrolling(Config.use_cyclic_scrolling);
 	Albums->CenteredCursor(Config.centered_cursor);
 	Albums->SetItemDisplayer(Display::Pairs);
 	Albums->SetGetStringFunction(StringPairToString);
 	
-	Dirs = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, "Directories", Config.main_color, brNone);
+	Dirs = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, Config.titles_visibility ? "Directories" : "", Config.main_color, brNone);
 	Dirs->HighlightColor(Config.active_column_color);
 	Dirs->CyclicScrolling(Config.use_cyclic_scrolling);
 	Dirs->CenteredCursor(Config.centered_cursor);
@@ -80,18 +80,19 @@ void TagEditor::Init()
 	
 	LeftColumn = Config.albums_in_tag_editor ? Albums : Dirs;
 	
-	TagTypes = new Menu<std::string>(MiddleColumnStartX, MainStartY, MiddleColumnWidth, MainHeight, "Tag types", Config.main_color, brNone);
+	TagTypes = new Menu<std::string>(MiddleColumnStartX, MainStartY, MiddleColumnWidth, MainHeight, Config.titles_visibility ? "Tag types" : "", Config.main_color, brNone);
 	TagTypes->HighlightColor(Config.main_highlight_color);
 	TagTypes->CyclicScrolling(Config.use_cyclic_scrolling);
 	TagTypes->CenteredCursor(Config.centered_cursor);
 	TagTypes->SetItemDisplayer(Display::Generic);
 	
-	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	for (const SongInfo::Metadata *m = SongInfo::Tags; m->Name; ++m)
 		TagTypes->AddOption(m->Name);
 	TagTypes->AddSeparator();
 	TagTypes->AddOption("Filename");
 	TagTypes->AddSeparator();
-	TagTypes->AddOption("Options", 1, 1);
+	if (Config.titles_visibility)
+		TagTypes->AddOption("Options", 1, 1);
 	TagTypes->AddSeparator();
 	TagTypes->AddOption("Reset");
 	TagTypes->AddOption("Save");
@@ -99,7 +100,7 @@ void TagEditor::Init()
 	TagTypes->AddOption("Capitalize First Letters");
 	TagTypes->AddOption("lower all letters");
 	
-	Tags = new Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, "Tags", Config.main_color, brNone);
+	Tags = new Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, Config.titles_visibility ? "Tags" : "", Config.main_color, brNone);
 	Tags->HighlightColor(Config.main_highlight_color);
 	Tags->CyclicScrolling(Config.use_cyclic_scrolling);
 	Tags->CenteredCursor(Config.centered_cursor);
@@ -223,7 +224,7 @@ void TagEditor::Refresh()
 
 void TagEditor::Update()
 {
-	if (LeftColumn->Empty())
+	if (LeftColumn->ReallyEmpty())
 	{
 		LeftColumn->Window::Clear();
 		Tags->Clear();
@@ -232,6 +233,7 @@ void TagEditor::Update()
 		{
 			*Albums << XY(0, 0) << "Fetching albums...";
 			Albums->Window::Refresh();
+			Mpd.BlockIdle(1); // for the same reason as in media library
 			Mpd.GetList(list, MPD_TAG_ALBUM);
 			for (MPD::TagList::const_iterator it = list.begin(); it != list.end(); ++it)
 			{
@@ -246,6 +248,7 @@ void TagEditor::Update()
 				}
 				MPD::FreeSongList(l);
 			}
+			Mpd.BlockIdle(0);
 			Albums->Sort<CaseInsensitiveSorting>();
 		}
 		else
@@ -279,7 +282,7 @@ void TagEditor::Update()
 		TagTypes->Refresh();
 	}
 	
-	if (Tags->Empty())
+	if (Tags->ReallyEmpty())
 	{
 		Tags->Reset();
 		MPD::SongList list;
@@ -548,8 +551,8 @@ void TagEditor::EnterPressed()
 	
 	if (id < 11)
 	{
-		MPD::Song::GetFunction get = Info::Tags[id].Get;
-		MPD::Song::SetFunction set = Info::Tags[id].Set;
+		MPD::Song::GetFunction get = SongInfo::Tags[id].Get;
+		MPD::Song::SetFunction set = SongInfo::Tags[id].Set;
 		if (id > 0 && w == TagTypes)
 		{
 			LockStatusbar();
@@ -652,7 +655,7 @@ void TagEditor::EnterPressed()
 
 void TagEditor::SpacePressed()
 {
-	if (w == Tags)
+	if (w == Tags && !Tags->Empty())
 	{
 		Tags->Select(Tags->Choice(), !Tags->isSelected());
 		w->Scroll(wDown);
@@ -767,6 +770,8 @@ MPD::Song *TagEditor::CurrentSong()
 
 void TagEditor::GetSelectedSongs(MPD::SongList &v)
 {
+	if (w != Tags || Tags->Empty())
+		return;
 	std::vector<size_t> selected;
 	Tags->GetSelected(selected);
 	if (selected.empty())
@@ -804,7 +809,7 @@ void TagEditor::NextColumn()
 		w = TagTypes;
 		TagTypes->HighlightColor(Config.active_column_color);
 	}
-	else if (w == TagTypes && TagTypes->Choice() < 12 && !Tags->Empty())
+	else if (w == TagTypes && TagTypes->Choice() < 12 && !Tags->ReallyEmpty())
 	{
 		TagTypes->HighlightColor(Config.main_highlight_color);
 		w->Refresh();
@@ -877,7 +882,7 @@ void TagEditor::LocateSong(const MPD::Song &s)
 		Dirs->Reset(); // go to the first pos, which is "." (music dir root)
 	
 	// highlight directory we need and get files from it
-	std::string dir = ExtractTopDirectory(s.GetDirectory());
+	std::string dir = ExtractTopName(s.GetDirectory());
 	for (size_t i = 0; i < Dirs->Size(); ++i)
 	{
 		if ((*Dirs)[i].first == dir)
@@ -1067,7 +1072,7 @@ std::string TagEditor::CapitalizeFirstLetters(const std::string &s)
 
 void TagEditor::CapitalizeFirstLetters(MPD::Song &s)
 {
-	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	for (const SongInfo::Metadata *m = SongInfo::Tags; m->Name; ++m)
 	{
 		unsigned i = 0;
 		for (std::string tag; !(tag = (s.*m->Get)(i)).empty(); ++i)
@@ -1077,7 +1082,7 @@ void TagEditor::CapitalizeFirstLetters(MPD::Song &s)
 
 void TagEditor::LowerAllLetters(MPD::Song &s)
 {
-	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	for (const SongInfo::Metadata *m = SongInfo::Tags; m->Name; ++m)
 	{
 		unsigned i = 0;
 		for (std::string tag; !(tag = (s.*m->Get)(i)).empty(); ++i)
@@ -1101,7 +1106,7 @@ std::string TagEditor::TagToString(const MPD::Song &s, void *data)
 	std::string result;
 	size_t i = static_cast<Menu<std::string> *>(data)->Choice();
 	if (i < 11)
-		result = (s.*Info::Tags[i].Get)(0);
+		result = (s.*SongInfo::Tags[i].Get)(0);
 	else if (i == 12)
 		result = s.GetNewName().empty() ? s.GetName() : s.GetName() + " -> " + s.GetNewName();
 	return result.empty() ? Config.empty_tag : result;
