@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -535,7 +535,7 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 				if (x < maxx)
 				{
 					real_x++;
-					x += wcwidth((*tmp)[beginning+real_x-minx-1]);
+					x += std::max(wcwidth((*tmp)[beginning+real_x-minx-1]), 1);
 				}
 				else if (beginning < maxbeginning)
 					beginning++;
@@ -551,7 +551,7 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 				if (x > minx)
 				{
 					real_x--;
-					x -= wcwidth((*tmp)[beginning+real_x-minx]);
+					x -= std::max(wcwidth((*tmp)[beginning+real_x-minx]), 1);
 				}
 				else if (beginning > 0)
 					beginning--;
@@ -940,9 +940,11 @@ std::wstring ToWString(const std::string &s)
 
 size_t Window::Length(const std::wstring &ws)
 {
-	size_t length = 0;
-	for (std::wstring::const_iterator it = ws.begin(); it != ws.end(); ++it)
-		length += wcwidth(*it);
-	return length;
+#	ifdef WIN32
+	return ws.length();
+#	else
+	int len = wcswidth(ws.c_str(), -1);
+	return len < 0 ? ws.length() : len;
+#	endif // WIN32
 }
 
